{******************************************************************************}
{ Projeto: Componentes ACBr                                                    }
{  Biblioteca multiplataforma de componentes Delphi para interao com equipa- }
{ mentos de Automao Comercial utilizados no Brasil                           }
{                                                                              }
{ Direitos Autorais Reservados (c) 2020 Daniel Simoes de Almeida               }
{                                                                              }
{ Colaboradores nesse arquivo:  Jos M S Junior, Victor Hugo Gonzales          }
{                                                                              }
{  Voc pode obter a ltima verso desse arquivo na pagina do  Projeto ACBr    }
{ Componentes localizado em      http://www.sourceforge.net/projects/acbr      }
{                                                                              }
{  Esta biblioteca  software livre; voc pode redistribu-la e/ou modific-la }
{ sob os termos da Licena Pblica Geral Menor do GNU conforme publicada pela  }
{ Free Software Foundation; tanto a verso 2.1 da Licena, ou (a seu critrio) }
{ qualquer verso posterior.                                                   }
{                                                                              }
{  Esta biblioteca  distribuda na expectativa de que seja til, porm, SEM   }
{ NENHUMA GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU      }
{ ADEQUAO A UMA FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral Menor}
{ do GNU para mais detalhes. (Arquivo LICENA.TXT ou LICENSE.TXT)              }
{                                                                              }
{  Voc deve ter recebido uma cpia da Licena Pblica Geral Menor do GNU junto}
{ com esta biblioteca; se no, escreva para a Free Software Foundation, Inc.,  }
{ no endereo 59 Temple Street, Suite 330, Boston, MA 02111-1307 USA.          }
{ Voc tambm pode obter uma copia da licena em:                              }
{ http://www.opensource.org/licenses/lgpl-license.php                          }
{                                                                              }
{ Daniel Simes de Almeida - daniel@projetoacbr.com.br - www.projetoacbr.com.br}
{       Rua Coronel Aureliano de Camargo, 963 - Tatu - SP - 18270-170         }
{******************************************************************************}

{$I ACBr.inc}
unit ACBrBoletoWS;

interface

uses
  Classes,
  SysUtils,
  ACBrBoleto,
  pcnGerador,
  pcnLeitor,
  ACBrUtil.Strings,
  pcnConversao,
  synacode,
  synautil,
  ACBrJSON,
  ACBrBoletoConversao,
  ACBrBoletoRetorno,
  ACBrDFeSSL,
  dateutils,
  strutils,
  ACBrUtil.Base,
  ACBrUtil.FilesIO,
  ACBrUtil.XMLHTML,
  httpsend,
  ACBrBoletoWS.Rest.OAuth;

type

  EACBrBoletoWSException = class ( Exception );

  TBoletoWS = class;
  TRetornoEnvioClass = class;



  { TBoletoWSClass }
  TBoletoWSClass = class
  private
    FGerador: TGerador;
    FBoleto: TACBrBoleto;
    FTitulo: TACBrTitulo;
    FDFeSSL: TDFeSSL;
    FHTTPSend: THTTPSend;
    FBoletoWS: TBoletoWS;
    FRetornoBanco: TRetornoEnvioClass;
    FOAuth : TOAuth;

  protected

    FRetornoWS: String;
    FPDadosMsg: String;
    FTipoRegistro: String;

    function GerarRemessa: String; virtual;
    function Enviar: Boolean; virtual;

    property DFeSSL: TDFeSSL read FDFeSSL write FDFeSSL;
    property HTTPSend: THTTPSend read FHTTPSend write FHTTPSend;
    property BoletoWS: TBoletoWS read FBoletoWS;
    property Gerador: TGerador read FGerador;
    property Boleto: TACBrBoleto read FBoleto;
    property ATitulo: TACBrTitulo read FTitulo;
    property RetornoBanco: TRetornoEnvioClass read FRetornoBanco write FRetornoBanco;
    property OAuth: TOAuth read FOAuth write FOAuth;

  public
    constructor Create(ABoletoWS: TBoletoWS ); virtual;
    destructor  Destroy; Override;

  end;

  { BoletoWS }
  {$IFDEF RTL230_UP}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$ENDIF RTL230_UP}
  TBoletoWS = class( TACBrWebService )
  private
    FBanco: TACBrTipoCobranca;
    FBoletoWSClass: TBoletoWSClass;
    FBoleto: TACBrBoleto;
    FRetornoBanco: TRetornoEnvioClass;
    FRetornoWS: String;
    FArqLOG: String;

    procedure SetBanco(ABanco: TACBrTipoCobranca);
    procedure GravaLog(const AString : AnsiString );

    procedure Clear;

  protected

    property Banco: TACBrTipoCobranca read FBanco write SetBanco;

  public
    constructor Create(AOwner: TComponent);Override;
    destructor Destroy; override;
    procedure DoLog(const AString : String );
    function Enviar: Boolean; override;
    property RetornoBanco: TRetornoEnvioClass read FRetornoBanco;

  end;

  { TRetornoEnvioClass }
  {$IFDEF RTL230_UP}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64)]
  {$ENDIF RTL230_UP}

  TRetornoEnvioClass = class
  private
    FACBrBoleto: TACBrBoleto;
    FRetWS: String;
  	FEnvWS: String;
    FCodRetorno: Integer;
    FMsg: String;
    FLeitor: TLeitor;
    FHTTPResultCode : Integer;
  protected
    function LerListaRetorno: Boolean; virtual;
    function LerRetorno(const ARetornoWS: TACBrBoletoRetornoWS): Boolean; virtual;
    function RetornoEnvio(const AIndex: Integer): Boolean; virtual;

    property ACBrBoleto: TACBrBoleto read FACBrBoleto;
    property Leitor: TLeitor read FLeitor;
    property RetWS: String read FRetWS write FRetWS;
	  property EnvWs: String read FEnvWs;

  public
    constructor Create(ABoletoWS: TACBrBoleto); virtual;
    destructor  Destroy; Override;

    property Msg: String read FMsg write FMsg;
    property CodRetorno: Integer read FCodRetorno write FCodRetorno;
    property HTTPResultCode: Integer read FHTTPResultCode write FHTTPResultCode;

  end;



Const
  C_LER_RETORNO             = 'LerRetorno';
  C_LER_LISTA_RETORNO       = 'LerListaRetorno';
  C_RETORNO_ENVIO           = 'Retorno Envio';
  C_DFESSL                  = 'DFeSSL';
  C_OBTER_NOME_ARQUIVO      = 'Obter Nome Arquivo';
  C_DOLOG                   = 'DoLog';
  C_GERAR_REMESSA           = 'Gerar Remessa';
  C_ENVIAR                  = 'Enviar';
  C_REGISTRO_BOLETO         = 'registro_boleto';
  C_DEFINIR_SERVICO_EACTION = 'DefinirServicoEAction';
  C_DEFINIR_URL             = 'DefinirURL';
  c_DEFINIR_ENVELOPE_SOAP   = 'DefinirEnvelopeSoap';
  C_GERAR_HEADER            = 'GerarHeader';
  C_GERAR_DADOS             = 'GerarDados';
  C_DEFINIR_ROOT_ELEMENT    = 'RootElement';
  C_NO_CACHE                = 'no-cache';
  C_GRANT_TYPE              = 'grant_type';
  C_SCOPE                   = 'scope';
  C_CONTENT_TYPE            = 'Content-Type';
  C_CACHE_CONTROL           = 'Cache-Control';
  C_AUTHORIZATION           = 'Authorization';
  C_ACCESS_TOKEN            = 'access_token';
  C_ACCEPT                  = 'Accept';
  C_XML                     = 'xml';
  C_JSON                    = 'json';
  C_ID                      = 'id';

  C_ARQBOLETOWS_LOG  = 'ArqBoletoWS.log';
  C_RETORNO_REGISTRO = 'retorno_registro';
  C_ERRO             = 'erro';
  C_ERROR_CODE       = 'Error_Code ';
  C_HTTP_RESULT_CODE = 'HTTP_Result_Code ';

ResourceString
  S_METODO_NAO_IMPLEMENTADO       =  'Metodo %s nao Implementado ';
  S_OPERACAO_NAO_IMPLEMENTADO     =  'Operao %s nao Implementado para este Banco';
  S_ERRO_GERAR_TOKEN_AUTENTICACAO = 'Erro ao gerar token de Autenticao: %s';

implementation

uses
  ACBrBoletoW_Caixa,
  ACBrBoletoRet_Caixa,
  ACBrBoletoW_BancoBrasil,
  ACBrBoletoRet_BancoBrasil,
  ACBrBoletoW_BancoBrasil_API,
  ACBrBoletoRet_BancoBrasil_API,
  ACBrBoletoW_Itau,
  ACBrBoletoRet_Itau,
  ACBrBoletoW_Credisis,
  ACBrBoletoRet_Credisis,
  ACBrBoletoW_Sicredi_APIECOMM,
  ACBrBoletoRet_Sicredi_APIECOMM,
  ACBrBoletoW_Sicredi_APIV2,
  ACBrBoletoRet_Sicredi_APIV2,
  ACBrBoletoW_PenseBank_API,
  ACBrBoletoRet_PenseBank_API,
  ACBrBoletoW_Santander,
  ACBrBoletoRet_Santander,
  ACBrBoletoW_Inter_API,
  ACBrBoletoRet_Inter_API,
  ACBrBoletoW_Bancoob,
  ACBrBoletoRet_Bancoob;

{ TRetornoEnvioClass }

constructor TRetornoEnvioClass.Create(ABoletoWS: TACBrBoleto);
begin
  FRetWS          := '';
  FCodRetorno     := 0;
  FMsg            := '';
  FLeitor         := TLeitor.Create;
  FACBrBoleto     := ABoletoWS;

end;

destructor TRetornoEnvioClass.Destroy;
begin
  FLeitor.Free;
  inherited Destroy;
end;

function TRetornoEnvioClass.LerRetorno(const ARetornoWS: TACBrBoletoRetornoWS): Boolean;
begin
  Result := False;
  raise EACBrBoletoWSException.Create(ACBrStr(ClassName + Format( S_METODO_NAO_IMPLEMENTADO, [C_LER_RETORNO] )));

end;

function TRetornoEnvioClass.LerListaRetorno: Boolean;
begin
  Result := False;
  raise EACBrBoletoWSException.Create(ACBrStr(ClassName + Format( S_METODO_NAO_IMPLEMENTADO, [C_LER_LISTA_RETORNO] )));

end;

function TRetornoEnvioClass.RetornoEnvio(const AIndex: Integer): Boolean;
begin
  Result := False;
  raise EACBrBoletoWSException.Create(ACBrStr(ClassName + Format( S_METODO_NAO_IMPLEMENTADO, [C_RETORNO_ENVIO] )));

end;

{ TBoletoWSClass }

constructor TBoletoWSClass.Create(ABoletoWS: TBoletoWS);
begin
  FTipoRegistro := C_XML;
  FBoletoWS     := ABoletoWS;
  FGerador      := TGerador.Create;
  FHTTPSend     := THTTPSend.Create;
  FTitulo       := nil;

  if Assigned( ABoletoWS.FBoleto.Configuracoes.WebService ) then
    FDFeSSL := TDFeSSL(ABoletoWS.FBoleto.Configuracoes.WebService);

  FOAuth := TOAuth.Create(FHTTPSend,
                          ABoletoWS.FBoleto);

end;

destructor TBoletoWSClass.Destroy;
begin
  FGerador.Free;
  FOAuth.Free;
  FHTTPSend.Destroy;
  inherited Destroy;
end;

function TBoletoWSClass.GerarRemessa: String;
begin
  Result := '';
  raise EACBrBoletoWSException.Create(ACBrStr(ClassName + Format( S_METODO_NAO_IMPLEMENTADO, [C_GERAR_REMESSA] )));
end;

function TBoletoWSClass.Enviar: Boolean;
begin
  Result := False;
  raise EACBrBoletoWSException.Create(ACBrStr(ClassName + Format( S_METODO_NAO_IMPLEMENTADO, [C_ENVIAR] )));
end;

{ TBoletoWS }

procedure TBoletoWS.SetBanco(ABanco: TACBrTipoCobranca);
begin
  if ABanco = FBanco then
    exit;

  if Assigned(FBoletoWSClass) then
    FreeAndNil(FBoletoWSClass);

  if Assigned(FRetornoBanco) then
    FreeAndNil(FRetornoBanco);

  case ABanco of
    cobSicred:
      begin
        if UpperCase(FBoleto.Configuracoes.WebService.VersaoDF) = 'V2' then
        begin //API V2 (NOVA 2022)
          FBoletoWSClass := TBoletoW_Sicredi_APIV2.Create(Self);
          FRetornoBanco  := TRetornoEnvio_Sicredi_APIV2.Create(FBoleto);
        end else
        begin //API ECOMM
          FBoletoWSClass := TBoletoW_Sicredi_APIECOMM.Create(Self);
          FRetornoBanco  := TRetornoEnvio_Sicredi_APIECOMM.Create(FBoleto);
        end;
      end;
    cobCaixaEconomica:
      begin
        FBoletoWSClass := TBoletoW_Caixa.Create(Self);
        FRetornoBanco  := TRetornoEnvio_Caixa.Create(FBoleto);
      end;
    cobBancoDoBrasilWS:
      begin
        FBoletoWSClass := TBoletoW_BancoBrasil.Create(Self);
        FRetornoBanco  := TRetornoEnvio_BancoBrasil.Create(FBoleto);
      end;
    cobBancoDoBrasilAPI:
      begin
        FBoletoWSClass := TBoletoW_BancoBrasil_API.Create(Self);
        FRetornoBanco  := TRetornoEnvio_BancoBrasil_API.Create(FBoleto);
      end;
    cobItau:
      begin
        FBoletoWSClass := TBoletoW_Itau.Create(Self);
        FRetornoBanco  := TRetornoEnvio_Itau.Create(FBoleto);
      end;
    cobCrediSIS:
      begin
        FBoletoWSClass := TBoletoW_Credisis.Create(Self);
        FRetornoBanco  := TRetornoEnvio_Credisis.Create(FBoleto);
      end;
    cobPenseBankAPI:
      begin
        FBoletoWSClass := TBoletoW_PenseBank_API.Create(Self);
        FRetornoBanco  := TRetornoEnvio_PenseBank_API.Create(FBoleto);
      end;
    cobSantander :
      begin
        FBoletoWSClass := TBoletoW_Santander.Create(Self);
        FRetornoBanco  := TRetornoEnvio_Santander.Create(FBoleto);
      end;
    cobBancoInter :
      begin
        FBoletoWSClass := TBoletoW_Inter_API.Create(Self);
        FRetornoBanco  := TRetornoEnvio_Inter_API.Create(FBoleto);
      end;
    cobBancoob :
      begin
        FBoletoWSClass := TBoletoW_Bancoob.Create(Self);
        FRetornoBanco  := TRetornoEnvio_Bancoob.Create(FBoleto);
      end;
  else
    FBoletoWSClass := TBoletoWSClass.Create(Self);
    FRetornoBanco := TRetornoEnvioClass.Create(FBoleto);
  end;
  FBoletoWSClass.FBoleto := FBoleto;
  FBanco := ABanco;
end;

constructor TBoletoWS.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  if Assigned(AOwner) and (AOwner is TACBrBoleto) then
    FBoleto := TACBrBoleto(AOwner)
  else
    FBoleto := Nil;

  if FBoleto.Configuracoes.Arquivos.LogRegistro and
     not DirectoryExists( FBoleto.Configuracoes.Arquivos.PathGravarRegistro ) then
      ForceDirectories( FBoleto.Configuracoes.Arquivos.PathGravarRegistro );

  if not (DirectoryExists( FBoleto.Configuracoes.Arquivos.PathGravarRegistro) ) then
    FArqLOG := C_ARQBOLETOWS_LOG
  else
    FArqLOG := PathWithDelim( FBoleto.Configuracoes.Arquivos.PathGravarRegistro ) + C_ARQBOLETOWS_LOG;

  Clear;
end;

procedure TBoletoWS.Clear;
begin
  if Assigned(FBoletoWSClass) then
    FBoletoWSClass.Free;

  FBoletoWSClass := TBoletoWSClass.Create(Self);
end;

procedure TBoletoWS.DoLog(const AString: String);
var
  Tratado: Boolean;
begin
  Tratado := False;
  if Assigned( FBoleto.Configuracoes.Arquivos.OnGravarLog ) then
    FBoleto.Configuracoes.Arquivos.OnGravarLog( AString, Tratado );

  if Tratado or FBoleto.Configuracoes.Arquivos.LogRegistro then
    GravaLog( AString );

end;

procedure TBoletoWS.GravaLog(const AString: AnsiString);
begin
  if (FArqLOG = '') then
    Exit;

  WriteLog( FArqLOG, FormatDateTime('dd/mm/yy hh:nn:ss:zzz',now) + ' - ' + AString );
end;

destructor TBoletoWS.Destroy;
begin
  if Assigned(FBoletoWSClass) then
    FreeAndNil(FBoletoWSClass);

  if Assigned(FRetornoBanco) then
    FreeAndNil(FRetornoBanco);

  inherited Destroy;

end;

function TBoletoWS.Enviar: Boolean;
var
  indice: Integer;
  LJsonEnvio : String;
begin
  Banco := FBoleto.Banco.TipoCobranca;
  Result := False;

  try
    if FBoleto.ListadeBoletos.Count > 0 then
    begin
      for indice:= 0 to Pred(FBoleto.ListadeBoletos.Count) do
      begin
        FBoletoWSClass.FTitulo := FBoleto.ListadeBoletos[indice];
        LJsonEnvio := FBoletoWSClass.GerarRemessa;
        Result     :=  FBoletoWSClass.Enviar;
        FRetornoWS := FBoletoWSClass.FRetornoWS;

        RetornoBanco.RetWS := FRetornoWS;
		    RetornoBanco.FEnvWs := LJsonEnvio;
        RetornoBanco.RetornoEnvio(indice);
      end;
    end
    else
    if (FBoleto.Configuracoes.WebService.Operacao in [tpConsulta]) then //Apenas Consulta Genrica no precisa carregar Titulo na Lista
    begin
      FBoletoWSClass.GerarRemessa;
      Result               := FBoletoWSClass.Enviar;
      FRetornoWS           := FBoletoWSClass.FRetornoWS;
      RetornoBanco.RetWS   := FRetornoWS;
      RetornoBanco.RetornoEnvio(0);
    end;
  except
    on E: Exception do
    begin
      if not (Assigned(FBoletoWSClass.RetornoBanco)) or ( ( FBoletoWSClass.RetornoBanco.CodRetorno = 0 ) and
             ( Trim( FBoletoWSClass.RetornoBanco.Msg ) = '' ) ) then
        DoLog('Falha Envio: ' + ACBrStr(E.Message))
      else
        DoLog('Erro Envio: ' + ACBrStr( IntToStr(FBoletoWSClass.RetornoBanco.CodRetorno)
                             + sLineBreak + FBoletoWSClass.RetornoBanco.Msg
                             + sLineBreak + E.Message ));
       Raise;
    end;
  end;
end;
end.


